package javawrapper;


/**
 * Tests the LightBehaviors class.
 * <p>Copyright: Copyright (c) Charles D. Canham 2003</p>
 * <p>Company: Institute of Ecosystem Studies</p>
 * @author Lora E. Murphy
 * @version 1.0
 *
 * <br>Edit history:
 * <br>------------------
 * <br>April 28, 2004: Submitted in beta version (LEM)
 */
public class TestLightBehaviors
    extends ModelTestCase {

  /** Tree population object for testing */
  private TreePopulation m_oPop = null;

  /** Does set up by creating the tree population object
   * @throws Exception Won't do it.
   */
  protected void setUp() throws Exception {
    super.setUp();

    try {
      //Create our tree population
      m_oPop = new TreePopulation(null);
      m_oPop.SetSpeciesNames(new String[] {"ACRU", "ACSA"});
    }
    catch (ModelException oErr) {
      throw new Exception(oErr.getMessage());
    }
  }

  /** Does tear down
   * @throws Exception Won't do it.
   */
  protected void tearDown() throws Exception {
    m_oPop = null;
    super.tearDown();
  }

  /**
   * Tests validate data.
   */
  public void testValidateData() {
    //Case:  all valid.
    GUIManager oManager = null;
    String sFileName = null;
    LightBehaviors oLight = null;
    TreePopulation oPop = null;
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile3();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      oPop = oManager.GetTreePopulation();
      oLight.ValidateData(oPop);
    }
    catch (ModelException oErr) {
      fail("Light validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Case:  number of azimuth angles for GLI light is negative.
    oLight.m_iNumGLIAziDiv.SetValue( -10);
    try {
      oLight.ValidateData(oPop);
      fail("Light validation failed - Case:  number of azimuth angles for GLI light is negative.");
    }
    catch (ModelException oErr) {
      ;
    }
    oLight.m_iNumGLIAziDiv.SetValue(10);

    //Case:  number of altitude angles for GLI light is negative.
    oLight.m_iNumGLIAltDiv.SetValue( -10);
    try {
      oLight.ValidateData(oPop);
      fail("Light validation failed - Case:  number of altitude angles for GLI light is negative.");
    }
    catch (ModelException oErr) {
      ;
    }
    oLight.m_iNumGLIAltDiv.SetValue(10);

    try {
      //Case:  GLI points file is loaded correctly
      sFileName = "test_points.txt";
      WriteGLIPointsFile(sFileName);
      oLight.m_sGLIPointsInFile.SetValue(sFileName);
      oLight.GetBehaviorByKey("points").m_bIsEnabled = true;
      oLight.ValidateData(oPop);
      assertEquals(3, oLight.mp_oPoints.size());
      Points oPoint = (Points) oLight.mp_oPoints.elementAt(0);
      assertEquals(10, oPoint.GetX(), 0.001);
      assertEquals(11, oPoint.GetY(), 0.001);
      assertEquals(1, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(1);
      assertEquals(12, oPoint.GetX(), 0.001);
      assertEquals(13, oPoint.GetY(), 0.001);
      assertEquals(2, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(2);
      assertEquals(14, oPoint.GetX(), 0.001);
      assertEquals(15, oPoint.GetY(), 0.001);
      assertEquals(3, oPoint.GetHeight(), 0.001);
    } catch (java.io.IOException oErr) {
      fail("Problem writing GLI points file.  Message: "+oErr.getMessage());
    } catch (ModelException oErr) {
      fail("Light validation failed with message " + oErr.getMessage());
    } finally {
      new java.io.File(sFileName).delete();
    }

    //Case:  min sun angle for GLI light is zero.

    //Case:  number of azimuth angles for quadrat light is negative.

    //Case:  number of altitude angles for quadrat light is negative.

    //Case:  min sun angle for quadrat light is zero.

    //Case:  number of azimuth angles for GLI map is negative.

    //Case:  number of altitude angles for GLI map is negative.

    //Case:  min sun angle for GLI map is zero.

    //Case:  beam fraction of global radiation is not a proportion.

    //Case:  clear sky transmission coefficient is zero.

    //Case:  minimum canopy trees for storm light is less than 0.
    try {

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile3();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      oPop = oManager.GetTreePopulation();
      Behavior oBeh = oLight.GetBehaviorByKey("storm light");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.m_bIsEnabled = true;
      oLight.m_fStmLightMinCanopyTrees.SetValue((float)-0.4);
      oLight.ValidateData(oPop);
      fail("Light validation didn't catch negative minimum canopy trees.");
    }
    catch (ModelException oErr) {
      ;
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }

    //Case:  basal area light conifer "c" parameter = 0.
     try {
       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLFile3();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       Behavior oBeh = oLight.GetBehaviorByKey("basal area light");
       oBeh.m_bIsEnabled = true;
       oLight.m_fBasalAreaLightConiferC.SetValue((float)0);
       oLight.ValidateData(oPop);
       fail("Light validation didn't catch " + oLight.m_fBasalAreaLightConiferC.GetDescriptor() + " equal to 0.");
     }
     catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  basal area light angiosperm "c" parameter = 0.
     try {
       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLFile3();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       Behavior oBeh = oLight.GetBehaviorByKey("basal area light");
       oBeh.m_bIsEnabled = true;
       oLight.m_fBasalAreaLightAngiospermC.SetValue((float)0);
       oLight.ValidateData(oPop);
       fail("Light validation didn't catch " + oLight.m_fBasalAreaLightAngiospermC.GetDescriptor() + " equal to 0.");
     }
     catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  basal area light minimum DBH is less than 0.
     try {
       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLFile3();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       Behavior oBeh = oLight.GetBehaviorByKey("basal area light");
       oBeh.m_bIsEnabled = true;
       oLight.m_fBasalAreaLightMinDBH.SetValue(-10);
       oLight.ValidateData(oPop);
       fail("Light validation didn't catch " + oLight.m_fBasalAreaLightMinDBH.GetDescriptor() + " less than 0.");
     }
     catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  basal area light minimum change threshold is less than 0.
     try {
       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLFile3();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       Behavior oBeh = oLight.GetBehaviorByKey("basal area light");
       oBeh.m_bIsEnabled = true;
       oLight.m_fBasalAreaLightChangeThreshold.SetValue(-23);
       oLight.ValidateData(oPop);
       fail("Light validation didn't catch " + oLight.m_fBasalAreaLightChangeThreshold.GetDescriptor() + " less than 0.");
     }
     catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  "Basal Area Light" grid grid cell lengths don't divide evenly into
     //plot lengths.
     try {

       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLFile3();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       Behavior oBeh = oLight.GetBehaviorByKey("basal area light");
       oBeh.m_bIsEnabled = true;

       //int i;
       //boolean bFoundBasalAreaLightGrid = false;
       Grid oGrid = oManager.GetGridByName("Basal Area Light");
/*       for (i = 0; i < oLight.mp_oGrids.length; i++) {
         if (oLight.mp_oGrids[i].GetName().equals("Basal Area Light")) {
           oLight.mp_oGrids[i].SetXCellLength(9);
           bFoundBasalAreaLightGrid = true;
           break;
         }
       } */
       if (oGrid == null)
         fail("Couldn't find grid \"Basal Area Light\" to test grid cell lengths.");
       oGrid.SetXCellLength(9);
       oLight.ValidateData(oPop);
       fail("Light validation didn't catch Basal Area Light grid cell lengths that didn't divide into plot lengths.");

     } catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  constant GLI is not between 0 and 100.
     try {

       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLFile1();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       oLight.ValidateData(oPop);
       oLight.m_fConstantGLIValue.SetValue(-1);
       oLight.ValidateData(oPop);
       fail("Didn't catch constant GLI not between 0 and 100.");
     } catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  Average light enabled but not GLI Map.
     try {

       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLErrorFile1();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       oLight.ValidateData(oPop);
       fail("Didn't catch lack of GLI Map with Average Light.");
     } catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }

     //Case:  GLI Map not before Average light.
     try {

       oManager = new GUIManager(null);
       oManager.ClearCurrentData();
       sFileName = WriteXMLErrorFile2();
       oManager.InputXMLParameterFile(sFileName);
       oLight = oManager.GetLightBehaviors();
       oPop = oManager.GetTreePopulation();
       oLight.ValidateData(oPop);
       fail("Didn't catch GLI Map after Average Light.");
     } catch (ModelException oErr) {
       ;
     }
     catch (java.io.IOException oE) {
       fail("Caught IOException.  Message: " + oE.getMessage());
     }
  }

  /**
   * This makes sure that species change is handled gracefully.
   */
  public void testChangeOfSpecies() {
    GUIManager oManager = null;
    String sFileName = null;
    try {
      ModelEnum oEnum;

      oManager = new GUIManager(null);
      oManager.ClearCurrentData();
      sFileName = WriteAllLightXMLFile();
      oManager.InputXMLParameterFile(sFileName);
      LightBehaviors oLight = oManager.GetLightBehaviors();

      assertEquals(3, oLight.mp_oPoints.size());
      Points oPoint = (Points) oLight.mp_oPoints.elementAt(0);
      assertEquals(75, oPoint.GetX(), 0.001);
      assertEquals(80, oPoint.GetY(), 0.001);
      assertEquals(15, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(1);
      assertEquals(50, oPoint.GetX(), 0.001);
      assertEquals(50, oPoint.GetY(), 0.001);
      assertEquals(12, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(2);
      assertEquals(190, oPoint.GetX(), 0.001);
      assertEquals(193, oPoint.GetY(), 0.001);
      assertEquals(5, oPoint.GetHeight(), 0.001);
      assertEquals(((Float) oLight.mp_fLightTransmissionCoefficient.GetValue().elementAt(0)).floatValue(), 0.08, 0.001);
      assertEquals(((Float) oLight.mp_fLightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.48, 0.001);

      assertEquals(((Float) oLight.mp_fSnagClass1LightTransmissionCoefficient.GetValue().elementAt(0)).floatValue(), 0.02, 0.001);
      assertEquals(((Float) oLight.mp_fSnagClass1LightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.42, 0.001);

      assertEquals(((Float) oLight.mp_fSnagClass2LightTransmissionCoefficient.GetValue().elementAt(0)).floatValue(), 0.03, 0.001);
      assertEquals(((Float) oLight.mp_fSnagClass2LightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.43, 0.001);

      assertEquals(((Float) oLight.mp_fSnagClass3LightTransmissionCoefficient.GetValue().elementAt(0)).floatValue(), 0.04, 0.001);
      assertEquals(((Float) oLight.mp_fSnagClass3LightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.44, 0.001);

      oEnum = (ModelEnum) oLight.mp_iBasalAreaLightWhatType.GetValue().elementAt(0);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oLight.mp_iBasalAreaLightWhatType.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 0);

      String[] sNewSpecies = new String[] {
            "Species 3",
            "Species 2",
            "Species 1"};
      oManager.GetTreePopulation().SetSpeciesNames(sNewSpecies);
      oLight = oManager.GetLightBehaviors();

      assertEquals(3, oLight.mp_oPoints.size());
      oPoint = (Points) oLight.mp_oPoints.elementAt(0);
      assertEquals(75, oPoint.GetX(), 0.001);
      assertEquals(80, oPoint.GetY(), 0.001);
      assertEquals(15, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(1);
      assertEquals(50, oPoint.GetX(), 0.001);
      assertEquals(50, oPoint.GetY(), 0.001);
      assertEquals(12, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(2);
      assertEquals(190, oPoint.GetX(), 0.001);
      assertEquals(193, oPoint.GetY(), 0.001);
      assertEquals(5, oPoint.GetHeight(), 0.001);
      assertEquals(((Float) oLight.mp_fLightTransmissionCoefficient.GetValue().elementAt(2)).floatValue(), 0.08, 0.001);
      assertEquals(((Float) oLight.mp_fLightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.48, 0.001);

      assertEquals(((Float) oLight.mp_fSnagClass1LightTransmissionCoefficient.GetValue().elementAt(2)).floatValue(), 0.02, 0.001);
      assertEquals(((Float) oLight.mp_fSnagClass1LightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.42, 0.001);

      assertEquals(((Float) oLight.mp_fSnagClass2LightTransmissionCoefficient.GetValue().elementAt(2)).floatValue(), 0.03, 0.001);
      assertEquals(((Float) oLight.mp_fSnagClass2LightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.43, 0.001);

      assertEquals(((Float) oLight.mp_fSnagClass3LightTransmissionCoefficient.GetValue().elementAt(2)).floatValue(), 0.04, 0.001);
      assertEquals(((Float) oLight.mp_fSnagClass3LightTransmissionCoefficient.GetValue().elementAt(1)).floatValue(), 0.44, 0.001);

      int i;
      assertEquals(3, oLight.mp_iBasalAreaLightWhatType.GetValue().size());
      for (i = 0; i < 3; i++) {
        assertTrue(null != oLight.mp_iBasalAreaLightWhatType.GetValue().elementAt(i));
      }
      oEnum = (ModelEnum) oLight.mp_iBasalAreaLightWhatType.GetValue().elementAt(2);
      assertEquals(oEnum.GetValue(), 1);
      oEnum = (ModelEnum) oLight.mp_iBasalAreaLightWhatType.GetValue().elementAt(1);
      assertEquals(oEnum.GetValue(), 0);
    }
    catch (ModelException oErr) {
      fail("Light validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
    finally {
      new java.io.File(sFileName).delete();
    }
  }

  /**
   * Makes sure that light values are assigned correctly from the parameter
   * file for gli and quadrat light.  It makes sure they are cross-assigned
   * correctly, don't get mixed up, and don't get mixed up with other behaviors
   * which use light parameters.
   */
  public void testReadParFile() {
    GUIManager oManager = null;
    String sFileName = null;
    try {

      oManager = new GUIManager(null);

      //Case:  only gli light data.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile1();
      oManager.InputXMLParameterFile(sFileName);
      LightBehaviors oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_iHeightOfFishEyePhoto.GetValue(),
                   LightBehaviors.MID_CROWN);
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fMapMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumMapAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumMapAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fPointsMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumPointsAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumPointsAziDiv.GetValue(), 18);
      assertEquals( ( (Float) oLight.mp_fLightTransmissionCoefficient.GetValue().
                     elementAt(0)).floatValue(), 0.08, 0.001);

      //Backwards compatibility - constant GLI
      assertTrue(oLight.GetBehaviorByKey("constant gli").m_bIsEnabled);
      assertEquals(oLight.m_fConstantGLIValue.GetValue(), 100.0, 0.0001);

      new java.io.File(sFileName).delete();

      //Case:  only quadrat light data.
      sFileName = WriteXMLFile2();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_iHeightOfFishEyePhoto.GetValue(),
                   LightBehaviors.MID_CROWN);
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fMapMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumMapAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumMapAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fPointsMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumPointsAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumPointsAziDiv.GetValue(), 18);
      assertEquals( ( (Float) oLight.mp_fLightTransmissionCoefficient.GetValue().
                     elementAt(0)).floatValue(), 0.08, 0.001);
      new java.io.File(sFileName).delete();

      //Case:  quadrat, gli map, gli points, and gli light data.
      sFileName = WriteXMLFile3();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_iHeightOfFishEyePhoto.GetValue(),
                   LightBehaviors.MID_CROWN);
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(), (float) 0.5, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(), (float) 0.65, 0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), (float) 0.885, 0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 13);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 19);
      assertEquals(oLight.m_fMapMinSunAngle.GetValue(), (float) 0.985, 0.001);
      assertEquals(oLight.m_iNumMapAltDiv.GetValue(), 14);
      assertEquals(oLight.m_iNumMapAziDiv.GetValue(), 20);
      assertEquals(oLight.m_fMapLightHeight.GetValue(), 2.5, 0.001);
      assertEquals(oLight.m_fPointsMinSunAngle.GetValue(), (float) 1.985, 0.001);
      assertEquals(oLight.m_iNumPointsAltDiv.GetValue(), 15);
      assertEquals(oLight.m_iNumPointsAziDiv.GetValue(), 21);
      assertEquals(oLight.m_sGLIPointsOutFile.GetValue(), "test.txt");
      assertEquals(3, oLight.mp_oPoints.size());
      Points oPoint = (Points) oLight.mp_oPoints.elementAt(0);
      assertEquals(75, oPoint.GetX(), 0.001);
      assertEquals(80, oPoint.GetY(), 0.001);
      assertEquals(15, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(1);
      assertEquals(50, oPoint.GetX(), 0.001);
      assertEquals(50, oPoint.GetY(), 0.001);
      assertEquals(12, oPoint.GetHeight(), 0.001);
      oPoint = (Points) oLight.mp_oPoints.elementAt(2);
      assertEquals(190, oPoint.GetX(), 0.001);
      assertEquals(193, oPoint.GetY(), 0.001);
      assertEquals(5, oPoint.GetHeight(), 0.001);
      assertEquals(((Float) oLight.mp_fLightTransmissionCoefficient.GetValue().elementAt(0)).floatValue(), 0.08, 0.001);
      new java.io.File(sFileName).delete();

      //Case:  neither quadrat nor gli light data, but light-dependent seed survival.
      sFileName = WriteXMLFile4();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(), 0, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(), 0,
                   0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 0);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 0);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), 0, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 0);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 0);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), 0, 0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 0);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 0);
      assertTrue(oLight.mp_fLightTransmissionCoefficient.GetValue().size() == 0 ||
                 ( (Float) oLight.mp_fLightTransmissionCoefficient.GetValue().
                  elementAt(0)).floatValue() < 0.001);
      new java.io.File(sFileName).delete();

      //Case:  both quadrat and gli light data, as well as light-dependent seed survival.
      sFileName = WriteXMLFile5();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_iHeightOfFishEyePhoto.GetValue(),
                   LightBehaviors.MID_CROWN);
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), (float) 0.885,
                   0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 13);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 19);
      assertEquals( ( (Float) oLight.mp_fLightTransmissionCoefficient.GetValue().
                     elementAt(0)).floatValue(), 0.08, 0.001);
      EstablishmentBehaviors oEst = oManager.GetEstablishmentBehaviors();
      assertEquals(oEst.m_fLightHeight.GetValue(), (float) 2, 0.001);
      assertEquals(oEst.m_fBeamFractionOfGlobalRadiation.GetValue(), 0.6, 0.001);
      assertEquals(oEst.m_fClearSkyTransmissionCoefficient.GetValue(), 0.75,
                   0.001);
      assertEquals(oEst.m_iJulianDayGrowthStarts.GetValue(), 205);
      assertEquals(oEst.m_iJulianDayGrowthEnds.GetValue(), 358);
      assertEquals(oEst.m_fMinSunAngle.GetValue(), 0.985, 0.001);
      assertEquals(oEst.m_iNumAltDiv.GetValue(), 14);
      assertEquals(oEst.m_iNumAziDiv.GetValue(), 20);
      new java.io.File(sFileName).delete();

      //Case:  only gli map data.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile6();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fMapMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumMapAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumMapAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fPointsMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumPointsAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumPointsAziDiv.GetValue(), 18);
      assertEquals( ( (Float) oLight.mp_fLightTransmissionCoefficient.GetValue().
                     elementAt(0)).floatValue(), 0.08, 0.001);
      assertEquals(oLight.m_fMapLightHeight.GetValue(), 2, 0.001);
      new java.io.File(sFileName).delete();

      //Case:  only gli points data.
      oManager.ClearCurrentData();
      sFileName = WriteXMLFile7();
      oManager.InputXMLParameterFile(sFileName);
      oLight = oManager.GetLightBehaviors();
      assertEquals(oLight.m_fBeamFractionOfGlobalRadiation.GetValue(),
                   (float) 0.5, 0.001);
      assertEquals(oLight.m_fClearSkyTransmissionCoefficient.GetValue(),
                   (float) 0.65, 0.001);
      assertEquals(oLight.m_iJulianDayGrowthStarts.GetValue(), 105);
      assertEquals(oLight.m_iJulianDayGrowthEnds.GetValue(), 258);
      assertEquals(oLight.m_fGLIMinSunAngle.GetValue(), (float) 0.785, 0.001);
      assertEquals(oLight.m_iNumGLIAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumGLIAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fQuadratMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumQuadratAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumQuadratAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fMapMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumMapAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumMapAziDiv.GetValue(), 18);
      assertEquals(oLight.m_fPointsMinSunAngle.GetValue(), (float) 0.785,
                   0.001);
      assertEquals(oLight.m_iNumPointsAltDiv.GetValue(), 12);
      assertEquals(oLight.m_iNumPointsAziDiv.GetValue(), 18);
      assertEquals(3, oLight.mp_oPoints.size());
      assertEquals( ( (Float) oLight.mp_fLightTransmissionCoefficient.GetValue().
                     elementAt(0)).floatValue(), 0.08, 0.001);
      new java.io.File(sFileName).delete();

    }
    catch (ModelException oErr) {
      fail("Light validation failed with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Tests that the correct parameters are displayed for each behavior.
   */
  public void testFormatDataForDisplay() {
    try {
      GUIManager oManager = new GUIManager(null);
      LightBehaviors oLight = null;
      String[] p_sExpected;
      Behavior oBeh;

      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();

      //Case:  no light behaviors are enabled.
      assertEquals(null, oLight.FormatDataForDisplay(m_oPop));

      //Case:  only sail light is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("saillight");
      oBeh.m_bIsEnabled = true;
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      p_sExpected = new String[] {
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Calculated Crown Depth",
          "Sail Light Minimum Solar Angle, in degrees",
          "Sail Light Maximum Shading Neighbor Distance, in meters",
          "Height of Fisheye Photo",
          "First Day of Growing Season",
          "Last Day of Growing Season",
          "Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only constant GLI light is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("constant gli");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Constant GLI - Constant GLI Value (0-100)"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);


      //Case:  only gli light is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("gli");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Height of Fisheye Photo",
          "First Day of Growing Season",
          "Last Day of Growing Season",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for GLI Light, in rad",
          "Number of Azimuth Sky Divisions for GLI Light Calculations",
          "Number of Altitude Sky Divisions for GLI Light Calculations",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only quadrat gli light is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("quadrat");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Height of Fisheye Photo",
          "First Day of Growing Season",
          "Last Day of Growing Season",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for Quadrat Light, in rad",
          "Height at Which GLI is Calculated for Quadrats, in meters",
          "Number of Azimuth Sky Divisions for Quadrat Light Calculations",
          "Number of Altitude Sky Divisions for Quadrat Light Calculations",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Quadrat GLI - Always Calculate All GLIs",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only gli map is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("map");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "First Day of Growing Season",
          "Last Day of Growing Season",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for GLI Map Creator, in rad",
          "Height at Which GLI is Calculated for GLI Map, in meters",
          "Number of Azimuth Sky Divisions for GLI Map Creator Calculations",
          "Number of Altitude Sky Divisions for GLI Map Creator Calculations",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only gli points is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("points");
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "First Day of Growing Season",
          "Last Day of Growing Season",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for GLI Points Creator, in rad",
          "GLI Points Input File",
          "GLI Points Output File",
          "Number of Azimuth Sky Divisions for GLI Points Creator",
          "Number of Altitude Sky Divisions for GLI Points Creator",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only storm light is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("storm light");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Storm Light - Max Radius (m) for Damaged Neighbors",
          "Storm Light - Slope of Light Function",
          "Storm Light - Intercept of Light Function",
          "Storm Light - Standard Deviation",
          "Storm Light - Stochasticity",
          "Storm Light - Max Years Snags Affect Light",
          "Storm Light - Minimum Trees For Full Canopy",
          "Storm Light - Max Years Damaged Trees Affect Light"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  only density light is enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      oBeh = oLight.GetBehaviorByKey("basal area light");
      oBeh.AddSpeciesTypeCombo(new SpeciesTypeCombo(0, 3, m_oPop));
      oBeh.m_bIsEnabled = true;
      p_sExpected = new String[] {
          "Basal Area Light - Species Type",
          "Basal Area Light - Mean GLI \"a\" Parameter",
          "Basal Area Light - Conifer \"b\" Parameter",
          "Basal Area Light - Conifer \"c\" Parameter",
          "Basal Area Light - Angiosperm \"b\" Parameter",
          "Basal Area Light - Angiosperm \"c\" Parameter",
          "Basal Area Light - Lognormal PDF Sigma",
          "Basal Area Light - Minimum DBH for Trees",
          "Basal Area Light - Minimum BA Change for New GLI (m2)",
          "Basal Area Light - Search Radius for Neighbors (m)",
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      //Case:  all are enabled.
      oManager.ClearCurrentData();
      oManager.InputXMLParameterFile(WriteNoLightXMLFile1());
      oLight = oManager.GetLightBehaviors();
      int i;
      for (i = 0; i < oLight.mp_oChildBehaviors.length; i++) {
        oLight.mp_oChildBehaviors[i].m_bIsEnabled = true;
        if (!oLight.mp_oChildBehaviors[i].GetKey().equals("map") &&
        		!oLight.mp_oChildBehaviors[i].GetKey().equals("points"))
          oLight.mp_oChildBehaviors[i].AddSpeciesTypeCombo(
          		new SpeciesTypeCombo(0, 3, m_oPop));
      }
      p_sExpected = new String[] {
          "Beam Fraction of Global Radiation",
          "Clear Sky Transmission Coefficient",
          "Height of Fisheye Photo",
          "First Day of Growing Season",
          "Last Day of Growing Season",
          "Amount Canopy Light Transmission (0-1)",
          "Minimum Solar Angle for Quadrat Light, in rad",
          "Height at Which GLI is Calculated for Quadrats, in meters",
          "Number of Azimuth Sky Divisions for Quadrat Light Calculations",
          "Number of Altitude Sky Divisions for Quadrat Light Calculations",
          "Minimum Solar Angle for GLI Light, in rad",
          "Number of Azimuth Sky Divisions for GLI Light Calculations",
          "Number of Altitude Sky Divisions for GLI Light Calculations",
          "Calculated Crown Depth",
          "Sail Light Minimum Solar Angle, in degrees",
          "Sail Light Maximum Shading Neighbor Distance, in meters",
          "Light Filter Light Transmission Coefficient",
          "Height of Light Filter, in m",
          "Minimum Solar Angle for GLI Map Creator, in rad",
          "Quadrat GLI - Always Calculate All GLIs",
          "Height at Which GLI is Calculated for GLI Map, in meters",
          "Number of Azimuth Sky Divisions for GLI Map Creator Calculations",
          "Number of Altitude Sky Divisions for GLI Map Creator Calculations",
          "Snag Age Class 1 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 2 Amount Canopy Light Transmission (0-1)",
          "Snag Age Class 3 Amount Canopy Light Transmission (0-1)",
          "Upper Age (Yrs) of Snag Light Transmission Class 1",
          "Upper Age (Yrs) of Snag Light Transmission Class 2",
          "Storm Light - Max Radius (m) for Damaged Neighbors",
          "Storm Light - Slope of Light Function",
          "Storm Light - Intercept of Light Function",
          "Storm Light - Standard Deviation",
          "Storm Light - Stochasticity",
          "Storm Light - Max Years Damaged Trees Affect Light",
          "Storm Light - Max Years Snags Affect Light",
          "Storm Light - Minimum Trees For Full Canopy",
          "Minimum Solar Angle for GLI Points Creator, in rad",
          "GLI Points Input File",
          "GLI Points Output File",
          "Number of Azimuth Sky Divisions for GLI Points Creator",
          "Number of Altitude Sky Divisions for GLI Points Creator",
          "Basal Area Light - Species Type",
          "Basal Area Light - Mean GLI \"a\" Parameter",
          "Basal Area Light - Conifer \"b\" Parameter",
          "Basal Area Light - Conifer \"c\" Parameter",
          "Basal Area Light - Angiosperm \"b\" Parameter",
          "Basal Area Light - Angiosperm \"c\" Parameter",
          "Basal Area Light - Lognormal PDF Sigma",
          "Basal Area Light - Minimum DBH for Trees",
          "Basal Area Light - Minimum BA Change for New GLI (m2)",
          "Basal Area Light - Search Radius for Neighbors (m)",
          "Constant GLI - Constant GLI Value (0-100)"
      };
      TestTable(oLight.FormatDataForDisplay(m_oPop), p_sExpected);

      System.out.println("FormatDataForDisplay succeeded for light.");
    }
    catch (ModelException oErr) {
      fail("Caught error with message " + oErr.getMessage());
    }
    catch (java.io.IOException oE) {
      fail("Caught IOException.  Message: " + oE.getMessage());
    }
  }

  /**
   * Writes a GLI points file
   * @throws IOException if there is a problem writing the file.
   * @param sFileName The file name to write to
   */
  protected void WriteGLIPointsFile(String sFileName) throws java.io.IOException {
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("X\tY\tHeight\n");
    oOut.write("10\t11\t1\n");
    oOut.write("12\t13\t2\n");
    oOut.write("14\t15\t3\n");
    oOut.close();
  }

  /**
   * Writes a file with only gli light settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>radiallight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</gliLight>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with only quadrat light settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  protected String WriteXMLFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</quadratGliLight>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with both quadrat and gli light settings, as well as many
   * other light behaviors in the "other" category.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  protected String WriteXMLFile3() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Map Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Point Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Basal Area Light</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Average Light</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</gliLight>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
    oOut.write("<li_numAltGrids>13</li_numAltGrids>");
    oOut.write("<li_numAziGrids>19</li_numAziGrids>");
    oOut.write("</quadratGliLight>");
    oOut.write("<gliMap>");
    oOut.write("<li_minSunAngle>0.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>14</li_numAltGrids>");
    oOut.write("<li_numAziGrids>20</li_numAziGrids>");
    oOut.write("<li_mapLightHeight>2.5</li_mapLightHeight>");
    oOut.write("</gliMap>");
    oOut.write("<gliPoint>");
    oOut.write("<li_minSunAngle>1.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>15</li_numAltGrids>");
    oOut.write("<li_numAziGrids>21</li_numAziGrids>");
    oOut.write("<li_GLIPointsFilename>test.txt</li_GLIPointsFilename>");
    oOut.write("<li_GLIPoint x=\"75\" y=\"80\" h=\"15\"/>");
    oOut.write("<li_GLIPoint x=\"50\" y=\"50\" h=\"12\"/>");
    oOut.write("<li_GLIPoint x=\"190\" y=\"193\" h=\"5\"/>");
    oOut.write("</gliPoint>");
    oOut.write("<lightOther>");
    oOut.write("<li_baLightA>12.4</li_baLightA>");
    oOut.write("<li_baConiferLightB>1.8</li_baConiferLightB>");
    oOut.write("<li_baConiferLightC>2.1</li_baConiferLightC>");
    oOut.write("<li_baAngiospermLightB>3.83</li_baAngiospermLightB>");
    oOut.write("<li_baAngiospermLightC>3.04</li_baAngiospermLightC>");
    oOut.write("<li_baLightSigma>0.82</li_baLightSigma>");
    oOut.write("<li_baLightChangeThreshold>0.05</li_baLightChangeThreshold>");
    oOut.write("<li_baLightMinDBH>10</li_baLightMinDBH>");
    oOut.write("<li_baTreeType>");
    oOut.write("<li_bttVal species=\"Species_1\">1</li_bttVal>");
    oOut.write("</li_baTreeType>");
    oOut.write("</lightOther>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with neither quadrat and gli light settings, but with
   * puerto rico establishment.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  protected String WriteXMLFile4() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<establishment>");
    oOut.write("<es_optimumGLI>");
    oOut.write("<es_ogVal species=\"Species_1\">87</es_ogVal>");
    oOut.write("</es_optimumGLI>");
    oOut.write("<es_highSlope>");
    oOut.write("<es_hsVal species=\"Species_1\">0.0098</es_hsVal>");
    oOut.write("</es_highSlope>");
    oOut.write("<es_lowSlope>");
    oOut.write("<es_lsVal species=\"Species_1\">0.0098</es_lsVal>");
    oOut.write("</es_lowSlope>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.22</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.6</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.4</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">1</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.6</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.75</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>205</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>358</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>14</li_numAltGrids>");
    oOut.write("<li_numAziGrids>20</li_numAziGrids>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with both quadrat and gli light settings, and pr
   * establishment as well.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  protected String WriteXMLFile5() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Light Dependent Seed Survival</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Seed\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</gliLight>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
    oOut.write("<li_numAltGrids>13</li_numAltGrids>");
    oOut.write("<li_numAziGrids>19</li_numAziGrids>");
    oOut.write("</quadratGliLight>");
    oOut.write("<establishment>");
    oOut.write("<es_optimumGLI>");
    oOut.write("<es_ogVal species=\"Species_1\">87</es_ogVal>");
    oOut.write("</es_optimumGLI>");
    oOut.write("<es_highSlope>");
    oOut.write("<es_hsVal species=\"Species_1\">0.0098</es_hsVal>");
    oOut.write("</es_highSlope>");
    oOut.write("<es_lowSlope>");
    oOut.write("<es_lsVal species=\"Species_1\">0.0098</es_lsVal>");
    oOut.write("</es_lowSlope>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.2</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<es_lightExtCoeffPartDmg>");
    oOut.write("<es_lecpdVal species=\"Species_1\">0.22</es_lecpdVal>");
    oOut.write("</es_lightExtCoeffPartDmg>");
    oOut.write("<es_lightExtCoeffFullDmg>");
    oOut.write("<es_lecfdVal species=\"Species_1\">0.9</es_lecfdVal>");
    oOut.write("</es_lightExtCoeffFullDmg>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.6</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.4</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">1</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("<es_lightHeight>2</es_lightHeight>");
    oOut.write("<li_beamFractGlobalRad>0.6</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.75</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>205</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>358</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>14</li_numAltGrids>");
    oOut.write("<li_numAziGrids>20</li_numAziGrids>");
    oOut.write("<li_snagAgeClass1>10</li_snagAgeClass1>");
    oOut.write("<li_snagAgeClass2>17</li_snagAgeClass2>");
    oOut.write("</establishment>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with only gli map settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 10, 2005: Created (LEM)
   */
  protected String WriteXMLFile6() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<grid gridName=\"GLI Map\">");
    oOut.write("<ma_floatCodes>");
    oOut.write("<ma_floatCode label=\"GLI\">0</ma_floatCode>");
    oOut.write("</ma_floatCodes>");
    oOut.write("<ma_plotLenX>200.0</ma_plotLenX>");
    oOut.write("<ma_plotLenY>200.0</ma_plotLenY>");
    oOut.write("<ma_lengthXCells>10.0</ma_lengthXCells>");
    oOut.write("<ma_lengthYCells>10.0</ma_lengthYCells>");
    oOut.write("</grid>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Map Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliMap>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_mapLightHeight>2</li_mapLightHeight>");
    oOut.write("</gliMap>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes a file with only gli point settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>January 7, 2005: Created (LEM)
   */
  protected String WriteXMLFile7() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Point Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliPoint>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_GLIPointsFilename>test.txt</li_GLIPointsFilename>");
    oOut.write("<li_GLIPoint x=\"75\" y=\"80\" h=\"15\"/>");
    oOut.write("<li_GLIPoint x=\"50\" y=\"50\" h=\"12\"/>");
    oOut.write("<li_GLIPoint x=\"190\" y=\"193\" h=\"5\"/>");
    oOut.write("</gliPoint>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
   * Writes the beginning of test parameter files.
   * @param oOut FileWriter Filestream to write to.
   * @throws IOException if there is a problem writing the file.
   */
  private void WriteValidateFilePart1(java.io.FileWriter oOut) throws java.io.
      IOException {
    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
  }

  /**
   * Writes a file with no light settings.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   *
   * <br>Edit history:
   * <br>------------------
   * <br>August 14, 2006: Created (LEM)
   */
  protected String WriteNoLightXMLFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>densityselfthinning</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Sapling\"/>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<mortality>");
    oOut.write("<mo_selfThinRadius>");
    oOut.write("<mo_strVal species=\"Species_1\">9</mo_strVal>");
    oOut.write("</mo_selfThinRadius>");
    oOut.write("<mo_minDensityForMort>");
    oOut.write("<mo_mdfmVal species=\"Species_1\">0</mo_mdfmVal>");
    oOut.write("</mo_minDensityForMort>");
    oOut.write("<mo_selfThinAsymptote>");
    oOut.write("<mo_staVal species=\"Species_1\">0.1019</mo_staVal>");
    oOut.write("</mo_selfThinAsymptote>");
    oOut.write("<mo_selfThinDiamEffect>");
    oOut.write("<mo_stdieVal species=\"Species_1\">0.5391</mo_stdieVal>");
    oOut.write("</mo_selfThinDiamEffect>");
    oOut.write("<mo_selfThinDensityEffect>");
    oOut.write("<mo_stdeeVal species=\"Species_1\">0.00000877</mo_stdeeVal>");
    oOut.write("</mo_selfThinDensityEffect>");
    oOut.write("</mortality>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

  /**
  * Writes a file with average light but no GLI Map behavior.
  * @return The file name.
  * @throws IOException if there is a problem writing the file.
  *
  * <br>Edit history:
  * <br>------------------
  * <br>September 11, 2006: Created (LEM)
  */
  protected String WriteXMLErrorFile1() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    WriteValidateFilePart1(oOut);
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Average Light</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("</gliLight>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
    oOut.write("<li_numAltGrids>13</li_numAltGrids>");
    oOut.write("<li_numAziGrids>19</li_numAziGrids>");
    oOut.write("</quadratGliLight>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }

 /**
  * Writes a file with the GLI map behavior after the average light behavior.
  * @return The file name.
  * @throws IOException if there is a problem writing the file.
  *
  * <br>Edit history:
  * <br>------------------
  * <br>January 7, 2005: Created (LEM)
  */
  protected String WriteXMLErrorFile2() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
     java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

     WriteValidateFilePart1(oOut);
     oOut.write("<behaviorList>");
     oOut.write("<behavior>");
     oOut.write("<behaviorName>glilight</behaviorName>");
     oOut.write("<version>1</version>");
     oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
     oOut.write("</behavior>");
     oOut.write("<behavior>");
     oOut.write("<behaviorName>quadratlight</behaviorName>");
     oOut.write("<version>1</version>");
     oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
     oOut.write("</behavior>");
     oOut.write("<behavior>");
     oOut.write("<behaviorName>Average Light</behaviorName>");
     oOut.write("<version>1</version>");
     oOut.write("</behavior>");
     oOut.write("<behavior>");
     oOut.write("<behaviorName>GLI Map Creator</behaviorName>");
     oOut.write("<version>1</version>");
     oOut.write("</behavior>");
     oOut.write("</behaviorList>");
     oOut.write("<gliLight>");
     oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
     oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
     oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
     oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
     oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
     oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
     oOut.write("<li_numAltGrids>12</li_numAltGrids>");
     oOut.write("<li_numAziGrids>18</li_numAziGrids>");
     oOut.write("<li_lightExtinctionCoefficient>");
     oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
     oOut.write("</li_lightExtinctionCoefficient>");
     oOut.write("</gliLight>");
     oOut.write("<quadratGliLight>");
     oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
     oOut.write("<li_numAltGrids>13</li_numAltGrids>");
     oOut.write("<li_numAziGrids>19</li_numAziGrids>");
     oOut.write("</quadratGliLight>");
     oOut.write("<gliMap>");
     oOut.write("<li_minSunAngle>0.985</li_minSunAngle>");
     oOut.write("<li_numAltGrids>14</li_numAltGrids>");
     oOut.write("<li_numAziGrids>20</li_numAziGrids>");
     oOut.write("<li_mapLightHeight>2.5</li_mapLightHeight>");
     oOut.write("</gliMap>");
     oOut.write("</paramFile>");

     oOut.close();
     return sFileName;
  }

  /**
   * Writes a file with all light stuff, for testing species change.
   * @return The file name.
   * @throws IOException if there is a problem writing the file.
   */
  protected String WriteAllLightXMLFile() throws java.io.IOException {
    String sFileName = "\\testfile1.xml";
    java.io.FileWriter oOut = new java.io.FileWriter(sFileName);

    oOut.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>");
    oOut.write("<paramFile fileCode=\"06010101\">");
    oOut.write("<plot>");
    oOut.write("<timesteps>1</timesteps>");
    oOut.write("<yearsPerTimestep>1</yearsPerTimestep>");
    oOut.write("<randomSeed>1</randomSeed>");
    oOut.write("<plot_lenX>200.0</plot_lenX>");
    oOut.write("<plot_lenY>200.0</plot_lenY>");
    oOut.write("<plot_latitude>55.37</plot_latitude>");
    oOut.write("</plot>");
    oOut.write("<trees>");
    oOut.write("<tr_speciesList>");
    oOut.write("<tr_species speciesName=\"Species_1\" />");
    oOut.write("<tr_species speciesName=\"Species_2\" />");
    oOut.write("</tr_speciesList>");
    oOut.write("<tr_seedDiam10Cm>0.1</tr_seedDiam10Cm>");
    oOut.write("<tr_minAdultDBH>");
    oOut.write("<tr_madVal species=\"Species_1\">10.0</tr_madVal>");
    oOut.write("<tr_madVal species=\"Species_2\">10.0</tr_madVal>");
    oOut.write("</tr_minAdultDBH>");
    oOut.write("</trees>");
    oOut.write("<allometry>");
    oOut.write("<tr_canopyHeight>");
    oOut.write("<tr_chVal species=\"Species_1\">39.48</tr_chVal>");
    oOut.write("<tr_chVal species=\"Species_2\">39.48</tr_chVal>");
    oOut.write("</tr_canopyHeight>");
    oOut.write("<tr_stdAsympCrownRad>");
    oOut.write("<tr_sacrVal species=\"Species_1\">0.0549</tr_sacrVal>");
    oOut.write("<tr_sacrVal species=\"Species_2\">0.0549</tr_sacrVal>");
    oOut.write("</tr_stdAsympCrownRad>");
    oOut.write("<tr_stdCrownRadExp>");
    oOut.write("<tr_screVal species=\"Species_1\">1.0</tr_screVal>");
    oOut.write("<tr_screVal species=\"Species_2\">1.0</tr_screVal>");
    oOut.write("</tr_stdCrownRadExp>");
    oOut.write("<tr_conversionDiam10ToDBH>");
    oOut.write("<tr_cdtdVal species=\"Species_1\">0.8008</tr_cdtdVal>");
    oOut.write("<tr_cdtdVal species=\"Species_2\">0.8008</tr_cdtdVal>");
    oOut.write("</tr_conversionDiam10ToDBH>");
    oOut.write("<tr_stdAsympCrownHt>");
    oOut.write("<tr_sachVal species=\"Species_1\">0.389</tr_sachVal>");
    oOut.write("<tr_sachVal species=\"Species_2\">0.389</tr_sachVal>");
    oOut.write("</tr_stdAsympCrownHt>");
    oOut.write("<tr_stdCrownHtExp>");
    oOut.write("<tr_scheVal species=\"Species_1\">1.0</tr_scheVal>");
    oOut.write("<tr_scheVal species=\"Species_2\">1.0</tr_scheVal>");
    oOut.write("</tr_stdCrownHtExp>");
    oOut.write("<tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_sohdVal species=\"Species_1\">0.03418</tr_sohdVal>");
    oOut.write("<tr_sohdVal species=\"Species_2\">0.03418</tr_sohdVal>");
    oOut.write("</tr_slopeOfHeight-Diam10>");
    oOut.write("<tr_slopeOfAsymHeight>");
    oOut.write("<tr_soahVal species=\"Species_1\">0.0299</tr_soahVal>");
    oOut.write("<tr_soahVal species=\"Species_2\">0.0299</tr_soahVal>");
    oOut.write("</tr_slopeOfAsymHeight>");
    oOut.write("</allometry>");
    oOut.write("<behaviorList>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>glilight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>quadratlight</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("<applyTo species=\"Species_1\" type=\"Adult\"/>");
    oOut.write("<applyTo species=\"Species_2\" type=\"Adult\"/>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Map Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>GLI Point Creator</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Basal Area Light</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("<behavior>");
    oOut.write("<behaviorName>Average Light</behaviorName>");
    oOut.write("<version>1</version>");
    oOut.write("</behavior>");
    oOut.write("</behaviorList>");
    oOut.write("<gliLight>");
    oOut.write("<li_heightOfFishEyePhoto>0</li_heightOfFishEyePhoto>");
    oOut.write("<li_beamFractGlobalRad>0.5</li_beamFractGlobalRad>");
    oOut.write("<li_clearSkyTransCoeff>0.65</li_clearSkyTransCoeff>");
    oOut.write("<li_julianDayGrowthStarts>105</li_julianDayGrowthStarts>");
    oOut.write("<li_julianDayGrowthEnds>258</li_julianDayGrowthEnds>");
    oOut.write("<li_minSunAngle>0.785</li_minSunAngle>");
    oOut.write("<li_numAltGrids>12</li_numAltGrids>");
    oOut.write("<li_numAziGrids>18</li_numAziGrids>");
    oOut.write("<li_lightExtinctionCoefficient>");
    oOut.write("<li_lecVal species=\"Species_1\">0.08</li_lecVal>");
    oOut.write("<li_lecVal species=\"Species_2\">0.48</li_lecVal>");
    oOut.write("</li_lightExtinctionCoefficient>");
    oOut.write("<li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_s1lecVal species=\"Species_1\">0.02</li_s1lecVal>");
    oOut.write("<li_s1lecVal species=\"Species_2\">0.42</li_s1lecVal>");
    oOut.write("</li_snag1LightExtinctionCoefficient>");
    oOut.write("<li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_s2lecVal species=\"Species_1\">0.03</li_s2lecVal>");
    oOut.write("<li_s2lecVal species=\"Species_2\">0.43</li_s2lecVal>");
    oOut.write("</li_snag2LightExtinctionCoefficient>");
    oOut.write("<li_snag3LightExtinctionCoefficient>");
    oOut.write("<li_s3lecVal species=\"Species_1\">0.04</li_s3lecVal>");
    oOut.write("<li_s3lecVal species=\"Species_2\">0.44</li_s3lecVal>");
    oOut.write("</li_snag3LightExtinctionCoefficient>");
    oOut.write("</gliLight>");
    oOut.write("<quadratGliLight>");
    oOut.write("<li_minSunAngle>0.885</li_minSunAngle>");
    oOut.write("<li_numAltGrids>13</li_numAltGrids>");
    oOut.write("<li_numAziGrids>19</li_numAziGrids>");
    oOut.write("</quadratGliLight>");
    oOut.write("<gliMap>");
    oOut.write("<li_minSunAngle>0.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>14</li_numAltGrids>");
    oOut.write("<li_numAziGrids>20</li_numAziGrids>");
    oOut.write("<li_mapLightHeight>2.5</li_mapLightHeight>");
    oOut.write("</gliMap>");
    oOut.write("<gliPoint>");
    oOut.write("<li_minSunAngle>1.985</li_minSunAngle>");
    oOut.write("<li_numAltGrids>15</li_numAltGrids>");
    oOut.write("<li_numAziGrids>21</li_numAziGrids>");
    oOut.write("<li_GLIPointsFilename>test.txt</li_GLIPointsFilename>");
    oOut.write("<li_GLIPoint x=\"75\" y=\"80\" h=\"15\"/>");
    oOut.write("<li_GLIPoint x=\"50\" y=\"50\" h=\"12\"/>");
    oOut.write("<li_GLIPoint x=\"190\" y=\"193\" h=\"5\"/>");
    oOut.write("</gliPoint>");
    oOut.write("<lightOther>");
    oOut.write("<li_baLightA>12.4</li_baLightA>");
    oOut.write("<li_baConiferLightB>1.8</li_baConiferLightB>");
    oOut.write("<li_baConiferLightC>2.1</li_baConiferLightC>");
    oOut.write("<li_baAngiospermLightB>3.83</li_baAngiospermLightB>");
    oOut.write("<li_baAngiospermLightC>3.04</li_baAngiospermLightC>");
    oOut.write("<li_baLightSigma>0.82</li_baLightSigma>");
    oOut.write("<li_baLightChangeThreshold>0.05</li_baLightChangeThreshold>");
    oOut.write("<li_baLightMinDBH>10</li_baLightMinDBH>");
    oOut.write("<li_baTreeType>");
    oOut.write("<li_bttVal species=\"Species_1\">1</li_bttVal>");
    oOut.write("<li_bttVal species=\"Species_2\">0</li_bttVal>");
    oOut.write("</li_baTreeType>");
    oOut.write("</lightOther>");
    oOut.write("</paramFile>");

    oOut.close();
    return sFileName;
  }
}

